/*
 * Copyright (c) 2009, 2012, 2013, 2014 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Authors: Sugih Jamin, Manoj Rajagopalan
 * Example using vertex array only, without VBO nor VAO.
*/
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
using namespace std;

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

GLdouble width=1280.0;
GLdouble height=640.0;   /* window width and height */
int wd;                  /* GLUT window handle */

void
kbd(unsigned char key, int x, int y)
{
  switch((char)key) {
  case 'q':
  case 27:    /* ESC */
    glutDestroyWindow(wd);
    exit(0);
  default:
    break;
  }

  return;
}

void
reshape(int w, int h)
{
  width = (GLdouble) w;
  height = (GLdouble) h;

  glViewport(0, 0, (GLsizei) w, (GLsizei) h);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0.0, width, 0.0, height, -1.f, 1.f);

  return;
}

void
display(void)
{
  int x, y;

  GLfloat triad[3][2] = { // (0,0) is the lower left corner
    { 0.0, 64.0 }, 
    { 0.0, 0.0 }, 
    { 64.0, 0.0 } 
  }; 
  GLfloat quad[4][2] = { // (0,0) is the lower left corner
    { 0.0, 64.0 },  // v0
    { 0.0, 0.0 },   // v1
    { 64.0, 64.0 }, // v2; T0: v0-v1-v2
    { 64.0, 0.0 }   // v3; T1: v2-v1-v3
  }; 
  GLfloat colors[4][3] = {
    { 1.0, 0.0, 0.0 }, 
    { 0.0, 1.0, 0.0 }, 
    { 0.0, 0.0, 1.0 },
    { 0.0, 1.0, 1.0 }
  }; 

  glClearColor(1.0, 1.0, 1.0, 1.0);
  glClear(GL_COLOR_BUFFER_BIT);

  glColorPointer(3, GL_FLOAT, 0, colors);
  glMatrixMode(GL_MODELVIEW);
  for (y=0; y < 10; y++) {
    glLoadIdentity();
    glTranslatef(0.0, y*64.0, 0.0);
    for (x=0; x < 10; x++) {
      glVertexPointer(2, GL_FLOAT, 0, triad);
      glDrawArrays(GL_TRIANGLES, 0, 3);
      glTranslatef(64.0, 0.0, 0.0);
      glVertexPointer(2, GL_FLOAT, 0, quad);
      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
      glTranslatef(64.0, 0.0, 0.0);
    }
  }

  glFlush();

  return;
}

void
init_shapes(void)
{
  glEnableClientState(GL_VERTEX_ARRAY); // allow OpenGL server access to client memory
  glEnableClientState(GL_COLOR_ARRAY);  // allow OpenGL server access to client memory

  return;
}

int
main(int argc, char *argv[])
{
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_SINGLE | GLUT_RGBA);
  glutInitWindowSize((int) width, (int) height);
  wd = glutCreateWindow("Experiment with line drawing" /* title */ );
  
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutKeyboardFunc(kbd);
  
  init_shapes();
  
  glutMainLoop();
  
  exit(0);
}
