/*
 * Copyright (c) 2012 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Authors: Sugih Jamin
*/

var vaocva_spd;
var vaocva_canvas;
var vaocva_gl;
var triad_vao, quad_vao;

function vaocva_initWorld() {
    var vattributes = new Float32Array([
       0.0, 64.0,    // v0
       1.0, 0.0, 0.0,
       0.0, 0.0,     // v1
       0.0, 1.0, 0.0,
       64.0, 64.0,   // v2
       0.0, 0.0, 1.0,
       64.0, 0.0,    // v3
       0.0, 1.0, 1.0
    ]);

    var vPos = vaocva_gl.getAttribLocation(vaocva_spd, "vPos");
    var vCol = vaocva_gl.getAttribLocation(vaocva_spd, "vCol");

    var vbod = vaocva_gl.createBuffer();
    vaocva_gl.bindBuffer(vaocva_gl.ARRAY_BUFFER, vbod);
    vaocva_gl.bufferData(vaocva_gl.ARRAY_BUFFER, vattributes, vaocva_gl.DYNAMIC_DRAW);

    triad_vao = vaocva_gl.createVertexArray();
    vaocva_gl.bindVertexArray(triad_vao);
    vaocva_gl.enableVertexAttribArray(vPos);
    vaocva_gl.vertexAttribPointer(vPos, 2, vaocva_gl.FLOAT, false,
                           5*Float32Array.BYTES_PER_ELEMENT, 0);
    vaocva_gl.enableVertexAttribArray(vCol);
    vaocva_gl.vertexAttribPointer(vCol, 3, vaocva_gl.FLOAT, false,
                           5*Float32Array.BYTES_PER_ELEMENT, 2*Float32Array.BYTES_PER_ELEMENT);

    quad_vao = vaocva_gl.createVertexArray();
    vaocva_gl.bindVertexArray(quad_vao);
    vaocva_gl.enableVertexAttribArray(vPos);
    vaocva_gl.vertexAttribPointer(vPos, 2, vaocva_gl.FLOAT, false,
                           5*Float32Array.BYTES_PER_ELEMENT, 0);
    vaocva_gl.enableVertexAttribArray(vCol);
    vaocva_gl.vertexAttribPointer(vCol, 3, vaocva_gl.FLOAT, false,
                           5*Float32Array.BYTES_PER_ELEMENT, 2*Float32Array.BYTES_PER_ELEMENT);
}

function vaocva_drawWorld()
{
    var x, y;

    vaocva_gl.clearColor(1.0, 1.0, 1.0, 0.0);
    vaocva_gl.clear(vaocva_gl.COLOR_BUFFER_BIT);

    glumMatrixMode(GLUM_PROJECTION);
    glumLoadIdentity();
    glumOrtho2D(0.0, vaocva_canvas.width, 0.0, vaocva_canvas.height);
    glumShaderMatrix();

    glumMatrixMode(GLUM_MODELVIEW);
    for (y = 0; y < vaocva_canvas.clientHeight/64-1.0; y++) {
      glumLoadIdentity();
      glumTranslatef(0.0, y*64.0, 0.0);
      for (x = 0; x < vaocva_canvas.clientWidth/128-1.0; x++) {
        glumShaderMatrix();
        vaocva_gl.bindVertexArray(triad_vao);
        vaocva_gl.drawArrays(vaocva_gl.TRIANGLES, 0, 3);

        glumTranslatef(64.0, 0.0, 0.0);
        glumShaderMatrix();
        vaocva_gl.bindVertexArray(quad_vao);
        vaocva_gl.drawArrays(vaocva_gl.TRIANGLE_STRIP, 0, 4);

        glumTranslatef(64.0, 0.0, 0.0);
      }
    }

    vaocva_gl.flush();
}

function vaocva_reshape()
{
    glumResize();
    vaocva_drawWorld();
}

function vaocva_main(cvname, vsname, fsname)
{
    vaocva_canvas = glumInitDisplayMode(cvname, { alpha: false, depth: false });
    vaocva_gl = glumInitCanvasSize(0,0);
    vaocva_spd = glumInitShaders(vsname, fsname);
    glumInit();

    glumReshapeFunc(vaocva_reshape);

    vaocva_initWorld();
    vaocva_drawWorld();
}
